/* coblist.cpp -- generate class data list by extracting from binary COBs
 * (C) 1998 Eric Leif <ericleif@mindspring.com>
 *
 * Change Log:
 * Thursday, July 23, 1998 version 1.1 - changes from version 1.0
 *	- improved the routine to now account for multiple COBs using same class
 *	- purified the subDirectory function to make it more general purpose
 *
 * To do:
 *	- check the removers and see if it removes everything the COB injects
 *	- maybe rewrite it and do it properly, it seemed ideally suited for a large array
 *	  instead of some of the better C++ constructs, although it turned out to be a pain
 *	- send demons to chew the spines out of those people who used the class system incorrectly,
 *	  used Word(TM) files for their readme files, used non-descript names for their files, used
 *	  the word readme.txt as a name and those who didn't even include a readme period :/
 */


#include <windows.h>
#include <direct.h>
#include <stdio.h>
#include <stdlib.h>
#include <io.h>
#include <fcntl.h>
#include <ctype.h>
#include <string.h>
#include <malloc.h>


int subDirectory(char *);
int handleFile(char*, char*);
int parseCOB(char *, char *);
int parseTXT(char *, char *);


struct {
	char *COBName;
	char *contact;
	char events[100];
} COBList[5][16][257];



int
main(int argc, char *argv[])
{
	FILE *COBData;
	int family,genus,species,event;

	char *object[5][16]={
	{"System",'\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0'}, // 0
	{"Scenery",'\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0'}, // 1
	{"System",		// 2 1
	"Call Button",	// 2 2
	"Invisible",
	"Herb",
	"Egg",
	"Food",
	"Drink",
	"Vendor",
	"Music",
	"Animal",
	"Discomfort",
	"Comfort",
	"Toy",
	"Bigtoy",
	"Weed",},		// 2 15
	{"Mover",		// 3 1
	"Lift",
	"Computer",
	"Projector",
	"Cannon","Misc",'\0','\0','\0','\0','\0','\0','\0','\0','\0'},		// 3 5
	{"Norn",		// 4 1
	"Grendel",'\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0'}		// 4 2
	};

	char *intro=
"This file has been generated by coblist available at <http://www.mindspring.com/~ericleif/>.\n"
"Any class that has a comma separated list of COB names and/or email addresses those COBs\n"
"add scripts in that class, (possibly overwriting previous object scripts) the events listed\n"
"are all of those used by all the COBs named.\n"
"Any COB that is listed as species 256 has used the classification system incorrectly and is\n"
"not what the author thinks it is.\n"
"Any feedback about errors that are related to this generated file are welcome. (eg things that\n"
"are not based on the author of the COBs listed here)\n"
"\n\nFamily\tGenus\tSpecies\t\tCOB Name\t[event scripts used]\t<email>\n\n";


	subDirectory(".\\");
	if((COBData=fopen("COBClasses.txt", "w"))==NULL) {
		printf("error opening output file");
		return 1;
	}

	fprintf(COBData, "%s", intro);
	for(family=0;family<5;family++) {
		for(genus=0;genus<16;genus++) {
			for(species=0;species<257;species++) {
				if(COBList[family][genus][species].COBName!=NULL) {
					fprintf(COBData," %3d\t%3d\t%3d\t\t%s [ ",family,genus,species,
							COBList[family][genus][species].COBName);
					for(event=0;event<100;event++) {
						if(COBList[family][genus][species].events[event]=='1')
							fprintf(COBData, "%d ",event);
					}
					fprintf(COBData, "]");
					if(COBList[family][genus][species].contact!=NULL)
						fprintf(COBData,"  %s",COBList[family][genus][species].contact);
					fprintf(COBData, "\n");
				}
			}
		if(object[family][genus]!=NULL)
			fprintf(COBData, "\n%s\n", object[family][genus]);
		}
	}

	fclose(COBData);

	printf("\n<return>");
	getchar();

	return 0;
}


int
subDirectory(char *tmpDirectory)
{
	char lastDirectory[MAX_PATH];
	char nextDirectory[MAX_PATH];

	char directory[MAX_PATH];

	strcat((strcpy(directory,strcpy(lastDirectory, tmpDirectory))), "*.*");

	HANDLE handle;
	WIN32_FIND_DATA wfd;
	handle = FindFirstFile(directory, &wfd);

	while (handle!=INVALID_HANDLE_VALUE) {
		switch(wfd.dwFileAttributes) {
		case FILE_ATTRIBUTE_DIRECTORY:
			if(strcmp(wfd.cFileName, ".")&&strcmp(wfd.cFileName, "..")) {
				strcat(strcat(strcpy(nextDirectory, lastDirectory), wfd.cFileName), "\\");
				subDirectory(nextDirectory);
			}
			break;
		case FILE_ATTRIBUTE_ARCHIVE:
			handleFile(wfd.cFileName, lastDirectory);
			break;
		}
		if(!FindNextFile(handle, &wfd))
			break;
	}
	FindClose(handle);

	return 0;
}


int
handleFile(char *fullFilename, char *path)
{
	if((strstr(fullFilename, ".COB")||strstr(fullFilename, ".cob"))!=NULL) {
		char Filename[MAX_PATH];
		strcat(strcpy(Filename,path), fullFilename);
		printf("\n%s :%s\n", fullFilename,(parseCOB(Filename,fullFilename))?"Error":"Ok");
	}
	return 0;
}


int
parseCOB(char *filename, char *fname)
{
	printf("Parse'n COB");

	FILE *COBFile;
	char buffer[21];

	int scripts, Byte, hiByte, loByte;
	fpos_t pos;

	if((COBFile=fopen(filename, "r"))==NULL) {
		fclose(COBFile);
		return 1;
	}
	_setmode(_fileno(COBFile), _O_BINARY); // "rb" wasn't doing it, maybe a compiler prob

	if(fseek(COBFile, 16, 0)) {
		fclose(COBFile);
		return 1;
	}
	scripts=fgetc(COBFile);

	printf(":%d:",scripts);
	if(!scripts) {
		fclose(COBFile);
		return 1;
	}


	if(fseek(COBFile, 24, 0)) {
		fclose(COBFile);
		return 1;
	}


// get cob name without extension
	size_t length=strlen(fname);
	char *nameBuffer;
	if((nameBuffer=(char*)malloc(length-3))==NULL) {
		fclose(COBFile);
		return 1;
	}
	strncpy(nameBuffer, fname, length-4);	// remove extent
	nameBuffer[length-4]='\0';


	char email[100];
	email[0]='\0';
	if(parseTXT(filename, email))
		strcpy(email, "<none>");
	printf(" %s\n",email);

	while(scripts--){
		Byte=fgetc(COBFile);
		if(Byte==255) {
			loByte=fgetc(COBFile);
			hiByte=fgetc(COBFile);
			Byte=hiByte*256+loByte;
		}
		fgetpos(COBFile, &pos);

	{
		int tmpByte;
		fpos_t tmp=pos+3; // 'scr' ->p
		fsetpos(COBFile, &tmp);

		if((tmpByte=fgetc(COBFile))!='p') { // ignore scrx
			fclose(COBFile);
			return 1;
		}
		tmpByte=fgetc(COBFile);
	}

		int i;
		for(i=0;(buffer[i]=fgetc(COBFile))!=','||i>20;i++)
			;
		buffer[i]='\0';
		i=0;

		int fgse[4];
		char tmp[5];
		int k;

		for(int j=0;j<4;j++) {
			k=0;
			while(isdigit(buffer[i++]))
				tmp[k++]=buffer[i-1];
			tmp[k]='\0';
			fgse[j]=atoi(tmp);
			if(fgse[j]>255)		// special case for incorrect class numbers
				fgse[j]=256;
		}


		COBList[fgse[0]][fgse[1]][fgse[2]].events[fgse[3]]='1';

		if(COBList[fgse[0]][fgse[1]][fgse[2]].COBName==NULL) {
			if( (COBList[fgse[0]][fgse[1]][fgse[2]].COBName=(char*)malloc(strlen(nameBuffer)+1))!=NULL
				&&(COBList[fgse[0]][fgse[1]][fgse[2]].contact=(char*)malloc(strlen(email)+1))!=NULL ) {
				strcpy(COBList[fgse[0]][fgse[1]][fgse[2]].COBName, nameBuffer);
				strcpy(COBList[fgse[0]][fgse[1]][fgse[2]].contact, email);
			}else{
				fclose(COBFile);
				return 1;
			}
		}else{
			if((strstr(COBList[fgse[0]][fgse[1]][fgse[2]].COBName, nameBuffer)==NULL)) {
				size_t sizeN = strlen(COBList[fgse[0]][fgse[1]][fgse[2]].COBName)+strlen(nameBuffer)+3;
				size_t sizeE = strlen(COBList[fgse[0]][fgse[1]][fgse[2]].contact)+strlen(email)+3;
				char oldName[MAX_PATH];
				strcpy(oldName, COBList[fgse[0]][fgse[1]][fgse[2]].COBName);
				char oldEmail[100];
				strcpy(oldEmail, COBList[fgse[0]][fgse[1]][fgse[2]].contact);
				if( (COBList[fgse[0]][fgse[1]][fgse[2]].COBName=(char*)realloc((char*)COBList[fgse[0]][fgse[1]][fgse[2]].COBName, (sizeN) ))!=NULL
					&&(COBList[fgse[0]][fgse[1]][fgse[2]].contact=(char*)realloc((char*)COBList[fgse[0]][fgse[1]][fgse[2]].contact, (sizeE) ))!=NULL ) {
					strcpy(COBList[fgse[0]][fgse[1]][fgse[2]].COBName, oldName);
					strcat(COBList[fgse[0]][fgse[1]][fgse[2]].COBName, ", ");
					strcat(COBList[fgse[0]][fgse[1]][fgse[2]].COBName, nameBuffer);

					strcpy(COBList[fgse[0]][fgse[1]][fgse[2]].contact, oldEmail);
					strcat(COBList[fgse[0]][fgse[1]][fgse[2]].contact, ", ");
					strcat(COBList[fgse[0]][fgse[1]][fgse[2]].contact, email);
				}else{
					fclose(COBFile);
					return 1;
				}
			}
		}

		pos+=Byte;
		fsetpos(COBFile, &pos);
		printf("\n--- %s\n", buffer);
	}

	fclose(COBFile);

	return 0;
}


int
parseTXT(char *filename, char *emailAddress)
{
	printf("Parse'n text");
	FILE *COBInfo;

	HANDLE handle;
	WIN32_FIND_DATA info;
	int fileType=0,email=0;
	fpos_t posB, posE;
	int ch, chB, chE;

	char filenameBase[MAX_PATH];
	size_t length=strlen(filename);

	strncpy(filenameBase, filename, length-4);
	filenameBase[length-4]='\0';


	strcat(filenameBase, "*.*");


	handle = FindFirstFile(filenameBase, &info);

	while (handle!=INVALID_HANDLE_VALUE) {

		fileType=0;
		if((strstr(info.cFileName, ".TXT")||strstr(info.cFileName, ".txt"))!=NULL) {
			fileType=1;	// .txt
		}else{
			 if((strstr(info.cFileName, ".RTF")||strstr(info.cFileName, ".rtf"))!=NULL) {
				fileType=2; // .rtf
			}else{
				if((strstr(info.cFileName, ".DOC")||strstr(info.cFileName, ".doc"))!=NULL) {
					fileType=3; // .doc
				}
			}
		}

		if(fileType) {
			if((COBInfo=fopen(info.cFileName, "r"))==NULL) {
				printf("couldn't open info file");
				FindClose(handle);
				return 1;
			}

			_setmode(_fileno(COBInfo), _O_BINARY);

			int z=1;
			while(((ch=fgetc(COBInfo))!=EOF) && (!email)) {

				if(ch==64&&isalnum(chB)) {
					fgetpos(COBInfo, &posE);
					email++;
				}
				if(email&&!isalnum(chE=fgetc(COBInfo)))
					email--;
				chB=ch;
			}

			if(email) {
				fpos_t tmp=posE-1;

				int isOk=1;
				while(isOk) {
					tmp-=1;
					fsetpos(COBInfo, &tmp);
					if((ch=fgetc(COBInfo))<33||ch==':') {
						fgetpos(COBInfo, &posB);
						isOk--;
					}
					if(ch==EOF) {
						fclose(COBInfo);
						FindClose(handle);
						return 1;
					}
				}

				fsetpos(COBInfo, &posE);
				while((ch=fgetc(COBInfo))>32)
					;
				ungetc(ch, COBInfo);
				fgetpos(COBInfo, &posE);
				posE--;
				size_t mem=(size_t)posE-(size_t)posB;

				fsetpos(COBInfo, &posB);
				emailAddress[0]='<';
				int k=1;
				for(unsigned int j=0;j<=mem;j++) {
					ch=fgetc(COBInfo);
					if(isalnum(ch)||ch=='@'||ch=='.')
						emailAddress[k++]=ch;
				}
				emailAddress[k++]='>';
				emailAddress[k]='\0';

				FindClose(handle);
				fclose(COBInfo);
				return 0;

			}else{
				fclose(COBInfo);
				FindClose(handle);
				return 1;
			}

		}
		if(!FindNextFile(handle, &info)) {
			FindClose(handle);
			return 1;
		}
	}
	FindClose(handle);

	return 1;
}
